@@ -61,28 +61,28 @@ any_bib_duplicates <- function(bib.files, .report_error, rstudio = FALSE) {
     .[field != "abstract"] %>%
     dcast.data.table(formula = key ~ field, value.var = "value")
   
-  if ("origyear" %in% names(bibDT)){
+  if ("origyear" %in% names(bibDT)) {
     origyear <- NULL
     bibDT <- bibDT[is.na(origyear)]
   }
   
   date <- NULL
-  if ("date" %notin% names(bibDT)){
+  if ("date" %notchin% names(bibDT)) {
     bibDT[, date := NA_character_]
   }
   
   author <- NULL
-  if ("author" %notin% names(bibDT)){
+  if ("author" %notchin% names(bibDT)) {
     bibDT[, author := NA_character_]
   }
   
   title <- NULL
-  if ("title" %notin% names(bibDT)){
+  if ("title" %notchin% names(bibDT)) {
     bibDT[, title := NA_character_]
   }
   
   volume <- NULL
-  if ("volume" %notin% names(bibDT)){
+  if ("volume" %notchin% names(bibDT)) {
     bibDT[, volume := NA_character_]
   }
   
@@ -91,15 +91,12 @@ any_bib_duplicates <- function(bib.files, .report_error, rstudio = FALSE) {
     .[, Year := if_else(is.na(year),
                         if_else(is.na(date),
                                 NA_character_,
-                                substr(date, 0, 4)), 
+                                substr(date, 0L, 4L)), 
                         as.character(year))] %>%
     .[, Author := rev_forename_surname_bibtex(author)] %>%
     .[, Title := tolower(title)] %>%
     # ABS duplicate if identical without Australia
-    .[, Title := if_else(Author == "ABS", 
-                         
-                         gsub(", australia,", ",", Title, fixed = TRUE), 
-                         Title)]
+    .[Author == "ABS", Title := gsub(", australia,", ",", Title, fixed = TRUE)]
   
   
   
@@ -111,9 +108,12 @@ any_bib_duplicates <- function(bib.files, .report_error, rstudio = FALSE) {
       .[dups_tail | dups_head, .(key, Author, Title, date, year)] %>%
       .[order(Author, Title)]
     
-    stopifnot(nrow(DT_with_all_duplicates) %% 2 == 0, nrow(DT_with_all_duplicates) > 1)
+    stopifnot(nrow(DT_with_all_duplicates) %% 2L == 0L,
+              nrow(DT_with_all_duplicates) > 1L)
     
-    .report_error(line_no = NULL, context = "Possible duplicates in bibliographies.", error_message = "Possible duplicates in bibliography.")
+    .report_error(line_no = NULL,
+                  context = "Possible duplicates in bibliographies.",
+                  error_message = "Possible duplicates in bibliography.")
     
     for (dup in 1:(nrow(DT_with_all_duplicates) / 2)){
       if (dup == 6){
@@ -151,12 +151,12 @@ any_bib_duplicates <- function(bib.files, .report_error, rstudio = FALSE) {
     
     # If first author is dup in the same file, use that file,
     # else use the second file (where the dup is more likely?)
-    if (n_first_author_files == 1) {
+    if (n_first_author_files == 1L) {
       the_file <- first_author_with_dup[["bib_file"]]
       line_no <- first_author_with_dup[["line_no"]]
     } else {
-      the_file <- first_author_with_dup[["bib_file"]][2]
-      line_no <- first_author_with_dup[["line_no"]][2]
+      the_file <- first_author_with_dup[["bib_file"]][2L]
+      line_no <- first_author_with_dup[["line_no"]][2L]
     }
     
     print(authors_with_dups[(bad), .(bib_file, line_no, key, value, authors)])
