@@ -67,7 +67,11 @@ lint_bib <- function(bib_file, outfile = bib_file, leading_spaces = 2L){
   }
   
   # Add commas: 
-  out[is_field] <- gsub("\\}$", "\\},", out[is_field], perl = TRUE)
+  out[is_field & !startsWith(tolower(out), "@string")] <-
+    sub("\\}$",
+        "\\},",
+        out[is_field & !startsWith(tolower(out), "@string")],
+        perl = TRUE)
   
   writeLines(out, outfile, useBytes = TRUE)
 }
