@@ -79,9 +79,9 @@ replace_nth_LaTeX_argument <- function(tex_lines,
         # {} --> { }
         #         ^
         tex_lines_with_command_name_split[[el]] <-
-          c(tex_lines_with_command_name_split[[el]][seq.int(1, starts[row])], 
+          c(tex_lines_with_command_name_split[[el]][seq.int(1L, starts[row])], 
             .dummy_replacement,
-            tex_lines_with_command_name_split[[el]][-seq.int(1, starts[row])])
+            tex_lines_with_command_name_split[[el]][-seq.int(1L, starts[row])])
       }
     }
     
@@ -112,6 +112,10 @@ replace_nth_LaTeX_argument <- function(tex_lines,
 #' for performance.
 #' @export nth_arg_positions
 nth_arg_positions <- function(tex_lines, command_name, n = 1L, optional = FALSE, star = TRUE, data.tables = TRUE) {
+  if (!requireNamespace("stringi", quietly = TRUE)) {
+    warning("Using non-stringi method. Run install.packages('stringi') for tested use.")
+    return(extract_LaTeX_argument(tex_lines = tex_lines, command_name = command_name, n = n, optional = optional))
+  } else {
   Command_locations <-
     stringi::stri_locate_all_regex(str = tex_lines,
                                    # If command = \a, must not also detect \ab
@@ -213,6 +217,7 @@ nth_arg_positions <- function(tex_lines, command_name, n = 1L, optional = FALSE,
         by = .(command_no)] %>%
       unique(by = "command_no")
   }
+  }
 }
 
 
