@@ -11,6 +11,8 @@
 #' @param rstudio If available, should the report be allowed to modify the RStudio session (for example, to pop to the location of the error)?
 #' @param extra_cat_ante Character vector extra messages (placed before \code{context}).
 #' @param extra_cat_post Character vector extra messages (placed after \code{context}).
+#' @param silent (logical, default: \code{FALSE}) Suppress all output.
+#' @param halt Should failures halt via \code{stop} or just display a message in the console?
 #' @rdname report_error
 #' @export
 report2console <- function(file = NULL,
@@ -23,7 +25,9 @@ report2console <- function(file = NULL,
                            extra_cat_ante = NULL,
                            extra_cat_post = NULL,
                            rstudio = FALSE,
-                           log_file = NULL){
+                           log_file = NULL,
+                           silent = FALSE,
+                           halt = getOption("TeXCheckR.halt_on_error", FALSE)) {
   # Printing requirements:
   ## 1. Cross
   ## 2. Line no (if applicable)
@@ -33,14 +37,20 @@ report2console <- function(file = NULL,
   # crayon::red(NULL) -> Error in mypaste(...) need character strings
   Red <- function(x) if (!is.character(x)) x else red(x)
   bold_red <- function(x) if (!is.character(x)) x else bold(red(x))
-  cat("\n", 
-      bold_red(error_message), "\n",
-      bold_red(symbol$cross), " ", Red(line_no), ": ", unlist(extra_cat_ante), Red(context), unlist(extra_cat_post), "\n",
-      bold_red(advice), "\n",
-      sep = "")
-  
-  if (rstudio && !is.null(file) && rstudioapi::isAvailable()) {
-    rstudioapi::navigateToFile(file, line = line_no, column = if (is.null(column)) 1L else as.integer(column))
+  if (!silent && OR(getOption("TeXCheckR.capture.output", FALSE), !is_testing())) {
+    cat("\n", 
+        bold_red(error_message), "\n",
+        bold_red(symbol$cross), " ", Red(line_no), ": ",
+        unlist(extra_cat_ante), Red(context), unlist(extra_cat_post), "\n",
+        bold_red(advice), "\n",
+        sep = "")
+    
+    if (rstudio &&
+        !is.null(file) &&
+        interactive() &&
+        rstudioapi::isAvailable() && !is_testing()) {
+      rstudioapi::navigateToFile(file, line = line_no, column = if (is.null(column)) 1L else as.integer(column))
+    }
   }
   
   # To return the directory if applicable
@@ -71,6 +81,11 @@ report2console <- function(file = NULL,
                append = append)
     }, add = TRUE)
   }
+  
+  if (halt) {
+    stop(error_message)
+  }
+  
 }
 
 
